import processing.core.PApplet;
import toxi.geom.*;
import toxi.math.MathUtils;
import toxi.processing.ToxiclibsSupport;

public class Segment extends BoilerPlate {
	PApplet parent;

	public int seed = 0;
	public float seedNormal = 0.0f;

	public Ray3D currentRotator;
	public Ray3D currentArbitrary;
	public Ray3D currentTangent;

	public Vec3D currentDirection;
	public Ray3D current;

	public Vec3D previousDirection;
	public Ray3D previous;

	public Segment parentSegment;
	public float segmentScalar = 10.0f;

	public int frameCount;
	public float frameCountScalar;
	public float rotationScalar;
	public float seedNormalScalar;
	
	Segment(PApplet p) {
		parent = p;
		setScope("Segment");
		currentDirection = new Vec3D();
		current = new Ray3D();
		previousDirection = new Vec3D();
		previous = new Ray3D();

		currentArbitrary = new Ray3D(new Vec3D(),Vec3D.Y_AXIS);
		currentTangent = new Ray3D();
		currentRotator = new Ray3D();
		
	}

	public void randomizeCurrentControls() {
		float r;
		r = 1.0f;
		currentDirection = new Vec3D(getRandom(-r, r), getRandom(-r, r), getRandom(-r, r));
		r = 100.0f;
		current = new Ray3D(getRandom(-r, r), getRandom(-r, r), getRandom(-r, r), currentDirection);
	}

	public void setPreviousControls() {
		previousDirection = parentSegment.currentDirection;
		previous = parentSegment.current;
	}


	public void parse(ToxiclibsSupport gfx) {
		previousDirection.set(parentSegment.currentDirection);
		previous.set(parentSegment.current);

		current.set(current.sub(previous));
		current.normalizeTo(segmentScalar);
		current.addSelf(previous);
		current.setDirection(previous.sub(current));

		currentArbitrary.set(current);
		currentTangent.set(current);
		currentRotator.set(current);
		
	}

	public void draw(ToxiclibsSupport gfx) {
		// tracer("draw");
		parent.noFill();


		float colorAlpha = (1-seedNormal) * 255.0f + 0.0f;

		setLineStyle(1,0xFFFFFF00,1-seedNormal);
		gfx.ray(current,segmentScalar);

		int steps = 30;

		Vec3D currentVec = new Vec3D(current);
        Vec3D dirVec = new Vec3D(current.getDirection());
        Vec3D crossVec = currentArbitrary.cross(dirVec.getNormalized()).normalize();

        for (int i = 0; i < steps; i++) {
	        float percentage = (float)i/(float)steps;
	        float angleInRads = MathUtils.TWO_PI*percentage;
	        angleInRads += seedNormal*seedNormalScalar;	// extra offset of rotation
	        angleInRads += frameCount*frameCountScalar;	// speed of rotation
	        
	        Vec3D rotVec = crossVec.getRotatedAroundAxis(dirVec, angleInRads);
	        
	        currentRotator.setDirection(rotVec);
	        // setLineStyle(1,0xFFFF00FF,(1-seedNormal));
			// gfx.ray(currentRotator,rotationScalar);
	        
	        rotVec.scaleSelf(rotationScalar);
	        rotVec.addSelf(currentVec);
	        setLineStyle(1,0xFFFF00FF,(1-seedNormal));
			gfx.point(rotVec);


        }

	}

	public void setLineStyle(int weightParam,int colorParam,float alphaParam){
		parent.noFill();
		parent.strokeWeight(weightParam);
		parent.stroke(colorParam, 255.0f*alphaParam);

	}
}
